!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
!     ###########################
      MODULE MODD_CH_SNAP_n
!     ###########################
!
!!****  *MODD_CH_SNAP_n* - declaration of chemical emission data arrays
!!
!!    PURPOSE
!!    -------
!       The purpose of this declarative module is to specify  the 
!     chemical emission data arrays.
!
!!
!!**  IMPLICIT ARGUMENTS
!!    ------------------
!!      None 
!!
!!    REFERENCE
!!    ---------
!!      
!!
!!    AUTHOR
!!    ------
!!	D. Gazen   *L.A.*
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    08/03/2001                      
!!      01/12/03    (D.Gazen) change emissions handling for surf. externalization
!-------------------------------------------------------------------------------
!
!*       0.   DECLARATIONS
!             ------------
!
USE MODD_TYPE_EFUTIL
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
TYPE CH_EMIS_SNAP_t
!
  INTEGER            :: NEMIS_NBR
!                          ! number of chemical pgd fields chosen by user
  CHARACTER(LEN=3)                         :: CCONVERSION
!                          ! Unit conversion code
  CHARACTER(LEN=5)                         :: CSNAP_TIME_REF
!                          ! Reference time for Snap temporal profiles
!                          !  'UTC  ' : UTC   time
!                          !  'SOLAR' : SOLAR time
!                          !  'LEGAL' : LEGAL time
!                          !

  CHARACTER(LEN=6), DIMENSION(:), POINTER :: CEMIS_NAME
!                          ! name of the chemical fields (emitted species)
  CHARACTER(LEN=40), DIMENSION(:), POINTER :: CEMIS_COMMENT
!                          ! comment on the chemical fields (emitted species)
!
  REAL,     DIMENSION(:,:,:), POINTER:: XEMIS_FIELDS_SNAP ! Emission factor for
!                                                         ! each chemical specie and
!                                                         ! each snap
  REAL,     DIMENSION(:,:),   POINTER:: XEMIS_FIELDS      ! Emission for each specie
!                                                         ! (at a given time taking into 
!                                                         ! account all snaps)
  REAL,     DIMENSION(:),     POINTER:: XDELTA_LEGAL_TIME ! Difference (in hours)) between
!                                                         ! Legal time and UTC time
  INTEGER            :: NEMIS_SNAP                        ! number of snaps
  INTEGER            :: NSNAP_M                           ! number of months
  INTEGER            :: NSNAP_D                           ! number of days
  INTEGER            :: NSNAP_H                           ! number of hours
  REAL,              DIMENSION(:,:,:), POINTER:: XSNAP_MONTHLY
  REAL,              DIMENSION(:,:,:), POINTER:: XSNAP_DAILY
  REAL,              DIMENSION(:,:,:), POINTER:: XSNAP_HOURLY
  REAL,              DIMENSION(:),     POINTER:: XCONVERSION ! conversion factor
!
  TYPE(PRONOSVAR_T),               POINTER     :: TSPRONOSLIST ! Head pointer on pronostic
!                                                              variables list
LOGICAL  :: LEMIS_FIELDS ! Flag to know if emissions have been computed
!-------------------------------------------------------------------------------
!
END TYPE CH_EMIS_SNAP_t

TYPE(CH_EMIS_SNAP_t), ALLOCATABLE, TARGET, SAVE :: CH_EMIS_SNAP_MODEL(:)

INTEGER, POINTER :: NEMIS_NBR=>NULL()
!$OMP THREADPRIVATE(NEMIS_NBR)
 CHARACTER(LEN=3) ,               POINTER :: CCONVERSION=>NULL()
!$OMP THREADPRIVATE(CCONVERSION)
 CHARACTER(LEN=5) ,               POINTER :: CSNAP_TIME_REF=>NULL()
!$OMP THREADPRIVATE(CSNAP_TIME_REF)
 CHARACTER(LEN=40), DIMENSION(:), POINTER :: CEMIS_COMMENT=>NULL()
!$OMP THREADPRIVATE(CEMIS_COMMENT)
 CHARACTER(LEN=6), DIMENSION(:), POINTER :: CEMIS_NAME=>NULL()
!$OMP THREADPRIVATE(CEMIS_NAME)
REAL,              DIMENSION(:,:), POINTER:: XEMIS_FIELDS=>NULL()
!$OMP THREADPRIVATE(XEMIS_FIELDS)
REAL,              DIMENSION(:),   POINTER:: XDELTA_LEGAL_TIME=>NULL()
!$OMP THREADPRIVATE(XDELTA_LEGAL_TIME)
REAL,              DIMENSION(:,:,:), POINTER:: XEMIS_FIELDS_SNAP=>NULL()
!$OMP THREADPRIVATE(XEMIS_FIELDS_SNAP)
REAL,              DIMENSION(:,:,:), POINTER:: XSNAP_DAILY=>NULL()
!$OMP THREADPRIVATE(XSNAP_DAILY)
REAL,              DIMENSION(:,:,:), POINTER:: XSNAP_HOURLY=>NULL()
!$OMP THREADPRIVATE(XSNAP_HOURLY)
REAL,              DIMENSION(:,:,:), POINTER:: XSNAP_MONTHLY=>NULL()
!$OMP THREADPRIVATE(XSNAP_MONTHLY)
REAL,              DIMENSION(:), POINTER:: XCONVERSION=>NULL()
!$OMP THREADPRIVATE(XCONVERSION)
INTEGER, POINTER :: NEMIS_SNAP=>NULL()
!$OMP THREADPRIVATE(NEMIS_SNAP)
INTEGER, POINTER :: NSNAP_M=>NULL()
!$OMP THREADPRIVATE(NSNAP_M)
INTEGER, POINTER :: NSNAP_D=>NULL()
!$OMP THREADPRIVATE(NSNAP_D)
INTEGER, POINTER :: NSNAP_H=>NULL()
!$OMP THREADPRIVATE(NSNAP_H)
TYPE(PRONOSVAR_T),               POINTER     :: TSPRONOSLIST=>NULL()
!$OMP THREADPRIVATE(TSPRONOSLIST)
LOGICAL, POINTER :: LEMIS_FIELDS=>NULL()
!$OMP THREADPRIVATE(LEMIS_FIELDS)


CONTAINS

SUBROUTINE CH_EMIS_SNAP_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
CH_EMIS_SNAP_MODEL(KFROM)%CEMIS_COMMENT=>CEMIS_COMMENT
CH_EMIS_SNAP_MODEL(KFROM)%CEMIS_NAME=>CEMIS_NAME
CH_EMIS_SNAP_MODEL(KFROM)%XDELTA_LEGAL_TIME=>XDELTA_LEGAL_TIME
CH_EMIS_SNAP_MODEL(KFROM)%XEMIS_FIELDS=>XEMIS_FIELDS
CH_EMIS_SNAP_MODEL(KFROM)%XEMIS_FIELDS_SNAP=>XEMIS_FIELDS_SNAP
CH_EMIS_SNAP_MODEL(KFROM)%XSNAP_DAILY=>XSNAP_DAILY
CH_EMIS_SNAP_MODEL(KFROM)%XSNAP_HOURLY=>XSNAP_HOURLY
CH_EMIS_SNAP_MODEL(KFROM)%XSNAP_MONTHLY=>XSNAP_MONTHLY
CH_EMIS_SNAP_MODEL(KFROM)%XCONVERSION=>XCONVERSION
CH_EMIS_SNAP_MODEL(KFROM)%TSPRONOSLIST=>TSPRONOSLIST

ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_CH_SNAP_n:CH_EMIS_SNAP_GOTO_MODEL',0,ZHOOK_HANDLE)

NEMIS_NBR=>CH_EMIS_SNAP_MODEL(KTO)%NEMIS_NBR
CCONVERSION=>CH_EMIS_SNAP_MODEL(KTO)%CCONVERSION
CSNAP_TIME_REF=>CH_EMIS_SNAP_MODEL(KTO)%CSNAP_TIME_REF
CEMIS_COMMENT=>CH_EMIS_SNAP_MODEL(KTO)%CEMIS_COMMENT
CEMIS_NAME=>CH_EMIS_SNAP_MODEL(KTO)%CEMIS_NAME
XDELTA_LEGAL_TIME=>CH_EMIS_SNAP_MODEL(KTO)%XDELTA_LEGAL_TIME
XEMIS_FIELDS=>CH_EMIS_SNAP_MODEL(KTO)%XEMIS_FIELDS
XEMIS_FIELDS_SNAP=>CH_EMIS_SNAP_MODEL(KTO)%XEMIS_FIELDS_SNAP
XSNAP_DAILY=>CH_EMIS_SNAP_MODEL(KTO)%XSNAP_DAILY
XSNAP_HOURLY=>CH_EMIS_SNAP_MODEL(KTO)%XSNAP_HOURLY
XSNAP_MONTHLY=>CH_EMIS_SNAP_MODEL(KTO)%XSNAP_MONTHLY
XCONVERSION=>CH_EMIS_SNAP_MODEL(KTO)%XCONVERSION
TSPRONOSLIST=>CH_EMIS_SNAP_MODEL(KTO)%TSPRONOSLIST
NEMIS_SNAP=>CH_EMIS_SNAP_MODEL(KTO)%NEMIS_SNAP
NSNAP_M=>CH_EMIS_SNAP_MODEL(KTO)%NSNAP_M
NSNAP_D=>CH_EMIS_SNAP_MODEL(KTO)%NSNAP_D
NSNAP_H=>CH_EMIS_SNAP_MODEL(KTO)%NSNAP_H
LEMIS_FIELDS=>CH_EMIS_SNAP_MODEL(KTO)%LEMIS_FIELDS

IF (LHOOK) CALL DR_HOOK('MODD_CH_SNAP_n:CH_EMIS_SNAP_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE CH_EMIS_SNAP_GOTO_MODEL

SUBROUTINE CH_EMIS_SNAP_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_CH_SNAP_n:CH_EMIS_FIELD_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(CH_EMIS_SNAP_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(CH_EMIS_SNAP_MODEL(J)%CEMIS_COMMENT)
  NULLIFY(CH_EMIS_SNAP_MODEL(J)%CEMIS_NAME)
  NULLIFY(CH_EMIS_SNAP_MODEL(J)%XDELTA_LEGAL_TIME)
  NULLIFY(CH_EMIS_SNAP_MODEL(J)%XEMIS_FIELDS)
  NULLIFY(CH_EMIS_SNAP_MODEL(J)%XEMIS_FIELDS_SNAP)
  NULLIFY(CH_EMIS_SNAP_MODEL(J)%XSNAP_DAILY)
  NULLIFY(CH_EMIS_SNAP_MODEL(J)%XSNAP_HOURLY)
  NULLIFY(CH_EMIS_SNAP_MODEL(J)%XSNAP_MONTHLY)
  NULLIFY(CH_EMIS_SNAP_MODEL(J)%XCONVERSION)
ENDDO
CH_EMIS_SNAP_MODEL(:)%CCONVERSION=' '
CH_EMIS_SNAP_MODEL(:)%CSNAP_TIME_REF=' '
CH_EMIS_SNAP_MODEL(:)%NEMIS_NBR=0
CH_EMIS_SNAP_MODEL(:)%NEMIS_SNAP=0
CH_EMIS_SNAP_MODEL(:)%NSNAP_M=0
CH_EMIS_SNAP_MODEL(:)%NSNAP_D=0
CH_EMIS_SNAP_MODEL(:)%NSNAP_H=0
CH_EMIS_SNAP_MODEL(:)%LEMIS_FIELDS=.FALSE.
IF (LHOOK) CALL DR_HOOK("MODD_CH_SNAP_n:CH_EMIS_FIELD_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE CH_EMIS_SNAP_ALLOC

SUBROUTINE CH_EMIS_SNAP_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_CH_SNAP_n:CH_EMIS_FIELD_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(CH_EMIS_SNAP_MODEL)) DEALLOCATE(CH_EMIS_SNAP_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_CH_SNAP_n:CH_EMIS_FIELD_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE CH_EMIS_SNAP_DEALLO

END MODULE MODD_CH_SNAP_n

